
import Numeric

import geompy
import SALOMEDS
import salome
import smesh

import StdMeshers
import NETGENPlugin

from SMESH import *
from StdMeshers import *


## import GEOM dump file ## 
import string, os, sys, re


zeta = 0.5   # changed from 0.3 - to make it symmetric
zetam1 = 1-zeta
Bx = 3
By = 8
Bz = 23
block = geompy.MakeBoxDXDYDZ(Bx,By,Bz)


block26 = geompy.GetSubShape(block, [26])
p1star = geompy.MakeVertexOnCurve(block26,zeta)
block30 = geompy.GetSubShape(block, [30])
p2star = geompy.MakeVertexOnCurve(block30,zetam1)

# to be used in the mesher
x1,y1,z1 = geompy.PointCoordinates(p1star)
x2,y2,z2 = geompy.PointCoordinates(p2star)

DZplus = 17.0
DZmin  =  1.2 # create overlap of shell into volume block for Arlequin method
DZmin  =  0.0 # no overlap needed for LIAISON_MAIL, COQUE_MASSIF or MASSIF_COQUE


p1 = geompy.MakeVertexWithRef(p1star, 0, 0, -DZmin)
p2 = geompy.MakeVertexWithRef(p2star, 0, 0, -DZmin)
p3 = geompy.MakeVertexWithRef(p2star, 0, 0, DZplus)
p4 = geompy.MakeVertexWithRef(p1star, 0, 0, DZplus)

line1 = geompy.MakeLineTwoPnt(p1, p4)
line2 = geompy.MakeLineTwoPnt(p4, p3)
line3 = geompy.MakeLineTwoPnt(p3, p2)
line4 = geompy.MakeLineTwoPnt(p2, p1)
face1 = geompy.MakeFaceWires([line1, line2, line3, line4], 1)


# create groups (Faces) at top  and bottom of block
FBbot = geompy.CreateGroup(block, geompy.ShapeType["FACE"])
geompy.UnionIDs(FBbot, [31])
block = geompy.GetMainShape(FBbot)
FBtop = geompy.CreateGroup(block, geompy.ShapeType["FACE"])
geompy.UnionIDs(FBtop, [33])
block = geompy.GetMainShape(FBtop)


# create groups (edges) at top and bottom of face1
EFbot = geompy.CreateGroup(face1, geompy.ShapeType["EDGE"])
geompy.UnionIDs(EFbot, [6])
face1 = geompy.GetMainShape(EFbot)
EFtop = geompy.CreateGroup(face1, geompy.ShapeType["EDGE"])
geompy.UnionIDs(EFtop, [10])
face1 = geompy.GetMainShape(EFtop)

# create groups (shell and node) on faces face1-2
shell = geompy.CreateGroup(face1, geompy.ShapeType["FACE"])
geompy.UnionIDs(shell, [1])
face1 = geompy.GetMainShape(shell)

# make entities visible
geompy.addToStudy( p1, "p1" )
geompy.addToStudy( p2, "p2" )
geompy.addToStudy( p3, "p3" )
geompy.addToStudy( p4, "p4" )

geompy.addToStudy( line1, "line1" )
geompy.addToStudy( line2, "line2" )
geompy.addToStudy( line3, "line3" )
geompy.addToStudy( line4, "line4" )


# create compounds of constr <-- face1 and blcok
constr = geompy.MakeCompound([face1, block])

# define group on constr
Cline = geompy.CreateGroup(constr, geompy.ShapeType["EDGE"])
geompy.UnionIDs(Cline, [7])
constr = geompy.GetMainShape(Cline)
Ctop = geompy.CreateGroup(constr, geompy.ShapeType["FACE"])
geompy.UnionIDs(Ctop, [44])
constr = geompy.GetMainShape(Ctop)
Cbot = geompy.CreateGroup(constr, geompy.ShapeType["FACE"])
geompy.UnionIDs(Cbot, [42])
constr = geompy.GetMainShape(Cbot)

Nforce = geompy.CreateGroup(constr, geompy.ShapeType["VERTEX"])
geompy.UnionIDs(Nforce, [5, 10])
constr = geompy.GetMainShape(Nforce)


geompy.addToStudy( constr, "constr" )
geompy.addToStudy( face1, "face1" )
geompy.addToStudy( block, "block" )

geompy.addToStudyInFather(constr,block, "block" )
geompy.addToStudyInFather(constr,shell, "shell" )
geompy.addToStudyInFather(constr,FBtop, "FBtop" )
geompy.addToStudyInFather(constr,FBbot, "FBbot" )
geompy.addToStudyInFather(constr,Ctop, "Ctop" )
geompy.addToStudyInFather(constr,Cbot, "Cbot" )
geompy.addToStudyInFather(constr,Cline, "Cline" )
geompy.addToStudyInFather(constr,Nforce, "Nforce" )




### ============
### start mesher
### ============

constr = smesh.Mesh(constr)
NETGEN_3D_Parameters = smesh.CreateHypothesis('NETGEN_Parameters', 'NETGENEngine')
NETGEN_3D_Parameters.SetMaxSize( 0.4 )
NETGEN_3D_Parameters.SetSecondOrder( 1 )
NETGEN_3D_Parameters.SetOptimize( 1 )
NETGEN_3D_Parameters.SetFineness( 3 )
Max_Size3D = smesh.CreateHypothesis('MaxLength')
Max_Size3D.SetLength( 1.0 )
NETGEN_2D_Parameters = smesh.CreateHypothesis('NETGEN_Parameters_2D', 'NETGENEngine')
NETGEN_2D_Parameters.SetMaxSize( 0.4 )
NETGEN_2D_Parameters.SetSecondOrder( 0 )
NETGEN_2D_Parameters.SetOptimize( 1 )
NETGEN_2D_Parameters.SetFineness( 2 )
NETGEN_2D_Parameters.SetQuadAllowed( 0 )
Max_Size2D = smesh.CreateHypothesis('MaxLength')
Max_Size2D.SetLength( 1.0 )
Regular1D = constr.Segment()
Automatic_length = Regular1D.AutomaticLength(0.6)
#Quadratic_Mesh = Regular1D.QuadraticMesh()
MEFISTO_2D = constr.Triangle()
Tetrahedron_Netgen = constr.Tetrahedron(algo=smesh.NETGEN)
isDone = constr.Compute()


###=====================================
### add groups to mesh for loads, bc, materialprops
block = constr.Group(block)
shell = constr.Group(shell)
FBbot = constr.Group(FBbot)
FBtop = constr.Group(FBtop)
Cline = constr.Group(Cline)
Nforce = constr.Group(Nforce)



## set object names
smesh.SetName(constr.GetMesh(), 'constr')
smesh.SetName(NETGEN_3D_Parameters, 'NETGEN 3D Parameters')
smesh.SetName(Max_Size3D, 'Max Size 3D')
smesh.SetName(NETGEN_2D_Parameters, 'NETGEN 2D Parameters')
smesh.SetName(Max_Size2D, 'Max Size 2D')
smesh.SetName(Regular1D.GetAlgorithm(), 'Regular 1D')
smesh.SetName(Automatic_length, 'Automatic length')
#smesh.SetName(Quadratic_Mesh, 'Quadratic Mesh')
smesh.SetName(MEFISTO_2D.GetAlgorithm(), 'MEFISTO_2D')
smesh.SetName(Tetrahedron_Netgen.GetAlgorithm(), 'Tetrahedron (Netgen)')



smesh.SetName(block, 'block')
smesh.SetName(shell, 'shell')
smesh.SetName(FBbot, 'FBbot')
smesh.SetName(FBtop, 'FBtop')
smesh.SetName(Nforce, 'Nforce')


def PrintMeshInfo(constr):
     #aMesh = theMesh.GetMesh()
     print "mesh information:"
     print "Number of nodes       : ", constr.NbNodes()
     print "Number of edges       : ", constr.NbEdges()
     print "Number of faces       : ", constr.NbFaces()
     print "Number of volumes     : ", constr.NbVolumes()



number_of_volumes = constr.NbVolumes()
list_of_volumes = constr.GetElementsByType(VOLUME)   #ALL, NODE, EDGE, FACE, VOLUME (enumaration)
list_of_faces = constr.GetElementsByType(FACE)   #ALL, NODE, EDGE, FACE, VOLUME (enumaration)
PrintMeshInfo(constr)


# for point group:
if (0==1):
    refp = Numeric.array([Bx,By,Bz])  # centre of ball B
    BRadius  = 30          # radius of ball B
    ins,outs = node_group_near_point(constr,refp,BRadius)

# for line group:
# determine volume elements
if (0==1):
    p1 = Numeric.array([x1,y1,z1])
    p2 = Numeric.array([x2,y2,z2])
    VRadius = 0.8   #0.8
    Vnodesin,Vnodesout = node_group_near_line(constr,p1,p2,VRadius)

    grp = constr.CreateEmptyGroup(NODE, "VNin")
    grp.Add(Vnodesin) 
    grp = constr.CreateEmptyGroup(NODE, "VNout")
    grp.Add(Vnodesout) 

    Vinside = element_on_nodes(constr,list_of_volumes,Vnodesin)
    Voutside = element_exclude(list_of_volumes,Vinside)

    grp = constr.CreateEmptyGroup(VOLUME, "Vinside")
    grp.Add(Vinside) 
    grp = constr.CreateEmptyGroup(VOLUME, "Voutside")
    grp.Add(Voutside) 



# determine shell elements for Arlequin method
# these shell elements are submerged in the solid block elements
if (0==1):
    b1 = Numeric.array([0.495*Bx,0.01*By,Bz-2.000])  # min block
    b2 = Numeric.array([0.505*Bx,0.99*By,Bz-0.010])  # max block
    Bnin,Bnout = node_group_within_block(constr,b1,b2)
    Shin = element_on_nodes(constr,list_of_faces,Bnin)
    Shout = element_exclude(list_of_faces,Shin)

    grp = constr.CreateEmptyGroup(FACE, "Shin")
    grp.Add(Shin) 
    grp = constr.CreateEmptyGroup(FACE, "Shout")
    grp.Add(Shout) 


